package com.MythiCode.camera_kit_ext;

import android.Manifest;
import android.content.pm.PackageManager;
import android.os.Build;
import android.view.View;
import android.content.Context;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;

import io.flutter.embedding.engine.dart.DartExecutor;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.platform.PlatformView;
import android.net.Uri;

import android.graphics.BitmapFactory;
import android.graphics.Bitmap;
import com.google.gson.Gson;
import com.google.mlkit.vision.common.InputImage;
import com.google.mlkit.vision.text.Text;
import com.google.mlkit.vision.text.TextRecognition;
import com.google.mlkit.vision.text.TextRecognizer;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.common.util.concurrent.ListenableFuture;
import com.google.mlkit.vision.text.latin.TextRecognizerOptions;

import com.MythiCode.camera_kit_ext.Model.CornerPointModel;
import com.MythiCode.camera_kit_ext.Model.LineModel;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executor;
import android.graphics.Point;
import android.graphics.Rect;
import android.util.Log;

@RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
public class CameraKitExtFlutterView implements PlatformView, MethodChannel.MethodCallHandler, FlutterMethodListener {

    private static final int REQUEST_CAMERA_PERMISSION = 10001;
    private final MethodChannel channel;
    private final Context context;
    private final ActivityPluginBinding activityPluginBinding;
    private CameraBaseView cameraView;
    private String tag = "ANALYZER";
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final MethodChannel.Result result) {
        if (call.method.equals("requestPermission")) {
            if (ActivityCompat.checkSelfPermission(activityPluginBinding.getActivity(), Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
                ActivityCompat.requestPermissions(activityPluginBinding.getActivity(), new String[]{Manifest.permission.CAMERA}, REQUEST_CAMERA_PERMISSION);
                activityPluginBinding.addRequestPermissionsResultListener(new PluginRegistry.RequestPermissionsResultListener() {
                    @Override
                    public boolean onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
                        for (int i :
                                grantResults) {
                            if (i == PackageManager.PERMISSION_DENIED) {
                                try {
                                    result.success(false);
                                }catch (Exception e){

                                }
                                return false;
                            }
                        }
                        try {
                            result.success(true);
                        } catch (Exception e) {

                        }
                        return false;
                    }
                });
                return;
            } else {
                try {
                    result.success(true);
                } catch (Exception e) {

                }
            }
        } else if (call.method.equals("initCamera")) {
            boolean hasBarcodeReader = call.argument("hasBarcodeReader");
            char flashMode = call.argument("flashMode").toString().charAt(0);
            boolean isFillScale = call.argument("isFillScale");
            int barcodeMode = call.argument("barcodeMode");
            int androidCameraMode = call.argument("androidCameraMode");
            int cameraSelector = call.argument("cameraSelector");
            getCameraView().initCamera(hasBarcodeReader, flashMode, isFillScale, barcodeMode
                    , androidCameraMode, cameraSelector);
        } else if (call.method.equals("resumeCamera")) {
            getCameraView().resumeCamera();

        } else if (call.method.equals("pauseCamera")) {
            getCameraView().pauseCamera();
        } else if (call.method.equals("takePicture")) {
            String path = call.argument("path").toString();
            getCameraView().takePicture(path, result);
        } else if (call.method.equals("changeFlashMode")) {
            char captureFlashMode = call.argument("flashMode").toString().charAt(0);
            getCameraView().changeFlashMode(captureFlashMode);
        } else if (call.method.equals("dispose")) {
            dispose();
        } else if (call.method.equals("setCameraVisible")) {
            boolean isCameraVisible = call.argument("isCameraVisible");
            getCameraView().setCameraVisible(isCameraVisible);
        } else if (call.method.equals("processOCR")) {
            String path = call.argument("path").toString();
            processImageFromPathWithoutView(path,result);
//            result.success("111");
        }else {
            result.notImplemented();
        }
    }

    private CameraBaseView getCameraView() {
        return cameraView;
    }

    public CameraKitExtFlutterView(ActivityPluginBinding activityPluginBinding, DartExecutor dartExecutor, int viewId) {
        this.channel = new MethodChannel(dartExecutor, "plugins/camera_kit_" + viewId);
        this.activityPluginBinding = activityPluginBinding;
        this.channel.setMethodCallHandler(this);
        this.context = activityPluginBinding.getActivity().getApplicationContext();
        if (getCameraView() == null) {
            cameraView = new CameraBaseView(activityPluginBinding.getActivity(), this);
        }
    }

    @Override
    public View getView() {
        return getCameraView().getView();
    }

    @Override
    public void dispose() {
        if (getCameraView() != null) {
            getCameraView().dispose();
        }
    }

    @Override
    public void onBarcodeRead(String barcode) {
        channel.invokeMethod("onBarcodeRead", barcode);
    }

    @Override
    public void onTakePicture(final MethodChannel.Result result, final String filePath) {
        activityPluginBinding.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                result.success(filePath);
            }
        });
    }

    @Override
    public void onTakePictureFailed(final MethodChannel.Result result, final String errorCode, final String errorMessage) {
        activityPluginBinding.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                result.error(errorCode, errorMessage, null);
            }
        });
    }

    public void processImageFromPathWithoutView(final String p,final MethodChannel.Result flutterResult) {
        Log.d(tag, "processImageFromPathWithoutView");
    try {
//    BitmapFactory bitmap = BitmapFactory.decodeFile(path);
//    ExifInterface exif = ExifInterface(path)
//    val rotation = exif.getAttributeInt(
//            ExifInterface.TAG_ORIENTATION,
//            ExifInterface.ORIENTATION_NORMAL
//    )
//        Bitmap bitmap = BitmapFactory.decodeFile(p);
//        InputImage image = InputImage.fromBitmap(bitmap, 0);

        InputImage image = InputImage.fromFilePath(context
                , Uri.fromFile(new File(p)));

//      Bitmap bitmap = BitmapFactory.decodeFile(path);
//      InputImage image = InputImage.fromBitmap(bitmap, 0);
//
//    InputImage inputImage = InputImage.fromFilePath(activity
//            , Uri.fromFile(new File(path)));
        TextRecognizer recognizer = TextRecognition.getClient(TextRecognizerOptions.DEFAULT_OPTIONS);
//      TextRecognizer recognizer = TextRecognition.getClient();

        Task<Text> result = recognizer.process(image)
                        .addOnSuccessListener(new OnSuccessListener<Text>() {
                            @Override
                            public void onSuccess(Text visionText) {
                          Log.d(tag, "IT ${visionText.getText()}");
//                          Map<String, String> map = mutableMapOf();
//                          ArrayList<MutableMap<String, String>> listPoints = ArrayList();

//                          map.put("text",visionText.getText());
//                          map.put("path",path);
//                          map.put("orientation","0");
//                          map["text"] = visionText.getText();
//                          map["path"] = path;
//                          map["orientation"] = "0";
//

                                processText(visionText,p,flutterResult);
//                                Log.d(tag, visionText.getText);
//                                flutterResult.success(resutl);
                          for (Text.TextBlock block : visionText.getTextBlocks()) {

                            String blockText = block.getText();
                            Point[] blockCornerPoints = block.getCornerPoints();
                            Rect blockFrame = block.getBoundingBox();
                            for (Text.Line line : block.getLines()) {
                              String lineText = line.getText();
                              Point[] lineCornerPoints = line.getCornerPoints();
                              Rect lineFrame = line.getBoundingBox();
                              for (Text.Element element : line.getElements()) {
                                String elementText = element.getText();
//                                Map<String, String> value = mutableMapOf();
                                Point[] elementCornerPoints = element.getCornerPoints();
//                                value.put("text",elementText);
//                                value.put("cornerPoints",elementCornerPoints.toString());
//                                listPoints.add(value);
                                  Log.d(tag, elementText);
                                Rect elementFrame = element.getBoundingBox();
                              }
                            }
                          }



                                // Task completed successfully
                                // ...
                            }
                        })
                        .addOnFailureListener(
                                new OnFailureListener() {
                                    @Override
                                    public void onFailure(@NonNull Exception e) {
                                        Log.e(tag, "Failed to load the image "+e.toString());
//                                  Log.e(tag, "Failed to load the image ${e}");
                                        // Task failed with an exception
                                        // ...
                                    }
                                });

    } catch (IOException e) {
      e.printStackTrace();
      Log.e(tag, "Error to load the image From Path: $e");
    }
    }

    public void processText( Text text,String path,final MethodChannel.Result flutterResult) {
        List<LineModel> lineModels = new ArrayList<>();
        for (Text.TextBlock b : text.getTextBlocks())
            for (Text.Line line : b.getLines()) {
                LineModel lineModel = new LineModel(line.getText());
                for (Point p : line.getCornerPoints()) {
                    lineModel.cornerPoints.add(new CornerPointModel(p.x, p.y));
                }
                lineModels.add(lineModel);
            }
        Gson gson = new Gson();
        gson.toJson(lineModels);
//        return new Gson().toJson(lineModels);


        Map<String, Object> map = new HashMap<String, Object>();
        map.put("text",text.getText());
        map.put("values",lineModels);
        map.put("path",path);
        map.put("orientation",0);

        flutterResult.success(new Gson().toJson(map));
//        flutterResult.success(text.getText());
//        flutterResult.success(text.getText(),new Gson().toJson(lineModels), path);
    }
}
